@@ -100,7 +100,7 @@ G_BEGIN_DECLS
 typedef struct _GstEncodingTarget GstEncodingTarget;
 typedef GObjectClass GstEncodingTargetClass;
 
-GST_EXPORT
+GST_PBUTILS_API
 GType gst_encoding_target_get_type (void);
 
 /**
@@ -121,59 +121,57 @@ GType gst_encoding_target_get_type (void);
 #define gst_encoding_target_ref(target) \
   (g_object_ref ((GObject*) target))
 
-GST_EXPORT
+GST_PBUTILS_API
 GstEncodingTarget *     gst_encoding_target_new                 (const gchar *name,
                                                                  const gchar *category,
                                                                  const gchar *description,
                                                                  const GList *profiles);
 
-GST_EXPORT
+GST_PBUTILS_API
 const gchar *           gst_encoding_target_get_name            (GstEncodingTarget *target);
 
-GST_EXPORT
+GST_PBUTILS_API
 const gchar *           gst_encoding_target_get_category        (GstEncodingTarget *target);
 
-GST_EXPORT
+GST_PBUTILS_API
 const gchar *           gst_encoding_target_get_description     (GstEncodingTarget *target);
 
-GST_EXPORT
+GST_PBUTILS_API
 const GList *           gst_encoding_target_get_profiles        (GstEncodingTarget *target);
 
-GST_EXPORT
+GST_PBUTILS_API
 GstEncodingProfile *    gst_encoding_target_get_profile         (GstEncodingTarget *target,
                                                                  const gchar *name);
 
-GST_EXPORT
+GST_PBUTILS_API
 gboolean                gst_encoding_target_add_profile         (GstEncodingTarget *target,
                                                                  GstEncodingProfile *profile);
 
-GST_EXPORT
+GST_PBUTILS_API
 gboolean                gst_encoding_target_save                (GstEncodingTarget *target,
                                                                  GError **error);
 
-GST_EXPORT
+GST_PBUTILS_API
 gboolean                gst_encoding_target_save_to_file        (GstEncodingTarget *target,
                                                                  const gchar *filepath,
                                                                  GError **error);
 
-GST_EXPORT
+GST_PBUTILS_API
 GstEncodingTarget *     gst_encoding_target_load                (const gchar *name,
                                                                  const gchar *category,
                                                                  GError **error);
 
-GST_EXPORT
+GST_PBUTILS_API
 GstEncodingTarget *     gst_encoding_target_load_from_file      (const gchar *filepath,
                                                                  GError **error);
 
-GST_EXPORT
+GST_PBUTILS_API
 GList *                 gst_encoding_list_available_categories  (void);
 
-GST_EXPORT
+GST_PBUTILS_API
 GList *                 gst_encoding_list_all_targets           (const gchar * categoryname);
 
-#ifdef G_DEFINE_AUTOPTR_CLEANUP_FUNC
 G_DEFINE_AUTOPTR_CLEANUP_FUNC(GstEncodingTarget, gst_object_unref)
-#endif
 
 G_END_DECLS
 
