@@ -44,6 +44,51 @@
  * (or any other audio file for which decoders are installed) and re-encodes
  * it into an Ogg/Vorbis audio file.
  *
+ * A mix matrix can be passed to audioconvert, that will govern the
+ * remapping of input to output channels.
+ * ## Example matrix generation code
+ * To generate the matrix using code:
+ *
+ * |[
+ * GValue v = G_VALUE_INIT;
+ * GValue v2 = G_VALUE_INIT;
+ * GValue v3 = G_VALUE_INIT;
+ *
+ * g_value_init (&v2, GST_TYPE_ARRAY);
+ * g_value_init (&v3, G_TYPE_FLOAT);
+ * g_value_set_float (&v3, 1);
+ * gst_value_array_append_value (&v2, &v3);
+ * g_value_unset (&v3);
+ * [ Repeat for as many float as your input channels - unset and reinit v3 ]
+ * g_value_init (&v, GST_TYPE_ARRAY);
+ * gst_value_array_append_value (&v, &v2);
+ * g_value_unset (&v2);
+ * [ Repeat for as many v2's as your output channels - unset and reinit v2]
+ * g_object_set_property (G_OBJECT (audioconvert), "mix-matrix", &v);
+ * g_value_unset (&v);
+ * ]|
+ *
+ * ## Example launch line
+ * |[
+ * gst-launch-1.0 audiotestsrc ! audio/x-raw, channels=4 ! audioconvert mix-matrix="<<(float)1.0, (float)0.0, (float)0.0, (float)0.0>, <(float)0.0, (float)1.0, (float)0.0, (float)0.0>>" ! audio/x-raw,channels=2 ! autoaudiosink
+ * ]|
+ *
+ * > If an empty mix matrix is specified, a (potentially truncated)
+ * > identity matrix will be generated.
+ *
+ * ## Example empty matrix generation code
+ * |[
+ * GValue v = G_VALUE_INIT;
+ *
+ * g_value_init (&v, GST_TYPE_ARRAY);
+ * g_object_set_property (G_OBJECT (audioconvert), "mix-matrix", &v);
+ * g_value_unset (&v);
+ * ]|
+ *
+ * ## Example empty matrix launch line
+ * |[
+ * gst-launch-1.0 -v audiotestsrc ! audio/x-raw,channels=8 ! audioconvert mix-matrix="<>" ! audio/x-raw,channels=16,channel-mask=\(bitmask\)0x0000000000000000 ! fakesink
+ * ]|
  */
 
 /*
@@ -65,7 +110,6 @@
 #include <string.h>
 
 #include "gstaudioconvert.h"
-#include "plugin.h"
 
 GST_DEBUG_CATEGORY (audio_convert_debug);
 GST_DEBUG_CATEGORY_STATIC (GST_CAT_PERFORMANCE);
@@ -93,6 +137,8 @@ static gboolean gst_audio_convert_transform_meta (GstBaseTransform * trans,
     GstBuffer * outbuf, GstMeta * meta, GstBuffer * inbuf);
 static GstFlowReturn gst_audio_convert_submit_input_buffer (GstBaseTransform *
     base, gboolean is_discont, GstBuffer * input);
+static GstFlowReturn gst_audio_convert_prepare_output_buffer (GstBaseTransform *
+    base, GstBuffer * inbuf, GstBuffer ** outbuf);
 static void gst_audio_convert_set_property (GObject * object, guint prop_id,
     const GValue * value, GParamSpec * pspec);
 static void gst_audio_convert_get_property (GObject * object, guint prop_id,
@@ -110,6 +156,7 @@ enum
   PROP_0,
   PROP_DITHERING,
   PROP_NOISE_SHAPING,
+  PROP_MIX_MATRIX,
 };
 
 #define DEBUG_INIT \
@@ -123,7 +170,7 @@ G_DEFINE_TYPE_WITH_CODE (GstAudioConvert, gst_audio_convert,
 
 #define STATIC_CAPS \
 GST_STATIC_CAPS (GST_AUDIO_CAPS_MAKE (GST_AUDIO_FORMATS_ALL) \
-    ", layout = (string) interleaved")
+    ", layout = (string) { interleaved, non-interleaved }")
 
 static GstStaticPadTemplate gst_audio_convert_src_template =
 GST_STATIC_PAD_TEMPLATE ("src",
@@ -162,6 +209,17 @@ gst_audio_convert_class_init (GstAudioConvertClass * klass)
           GST_TYPE_AUDIO_NOISE_SHAPING_METHOD, GST_AUDIO_NOISE_SHAPING_NONE,
           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
+  g_object_class_install_property (gobject_class, PROP_MIX_MATRIX,
+      gst_param_spec_array ("mix-matrix",
+          "Input/output channel matrix",
+          "Transformation matrix for input/output channels",
+          gst_param_spec_array ("matrix-rows", "rows", "rows",
+              g_param_spec_float ("matrix-cols", "cols", "cols",
+                  -1, 1, 0,
+                  G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS),
+              G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS),
+          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+
   gst_element_class_add_static_pad_template (element_class,
       &gst_audio_convert_src_template);
   gst_element_class_add_static_pad_template (element_class,
@@ -186,8 +244,9 @@ gst_audio_convert_class_init (GstAudioConvertClass * klass)
       GST_DEBUG_FUNCPTR (gst_audio_convert_transform_meta);
   basetransform_class->submit_input_buffer =
       GST_DEBUG_FUNCPTR (gst_audio_convert_submit_input_buffer);
+  basetransform_class->prepare_output_buffer =
+      GST_DEBUG_FUNCPTR (gst_audio_convert_prepare_output_buffer);
 
-  basetransform_class->passthrough_on_same_caps = TRUE;
   basetransform_class->transform_ip_on_passthrough = FALSE;
 }
 
@@ -196,6 +255,7 @@ gst_audio_convert_init (GstAudioConvert * this)
 {
   this->dither = GST_AUDIO_DITHER_TPDF;
   this->ns = GST_AUDIO_NOISE_SHAPING_NONE;
+  g_value_init (&this->mix_matrix, GST_TYPE_ARRAY);
 
   gst_base_transform_set_gap_aware (GST_BASE_TRANSFORM (this), TRUE);
 }
@@ -210,6 +270,8 @@ gst_audio_convert_dispose (GObject * obj)
     this->convert = NULL;
   }
 
+  g_value_unset (&this->mix_matrix);
+
   G_OBJECT_CLASS (parent_class)->dispose (obj);
 }
 
@@ -239,47 +301,51 @@ parse_error:
   }
 }
 
-/* copies the given caps */
-static GstCaps *
-gst_audio_convert_caps_remove_format_info (GstCaps * caps, gboolean channels)
+static gboolean
+remove_format_from_structure (GstCapsFeatures * features,
+    GstStructure * structure, gpointer user_data G_GNUC_UNUSED)
 {
-  GstStructure *st;
-  gint i, n;
-  GstCaps *res;
-  guint64 channel_mask;
+  gst_structure_remove_field (structure, "format");
+  return TRUE;
+}
 
-  res = gst_caps_new_empty ();
+static gboolean
+remove_layout_from_structure (GstCapsFeatures * features,
+    GstStructure * structure, gpointer user_data G_GNUC_UNUSED)
+{
+  gst_structure_remove_field (structure, "layout");
+  return TRUE;
+}
 
-  n = gst_caps_get_size (caps);
-  for (i = 0; i < n; i++) {
-    gboolean remove_channels = FALSE;
+static gboolean
+remove_channels_from_structure (GstCapsFeatures * features, GstStructure * s,
+    gpointer user_data)
+{
+  guint64 mask;
+  gint channels;
+  GstAudioConvert *this = GST_AUDIO_CONVERT (user_data);
 
-    st = gst_caps_get_structure (caps, i);
-
-    /* If this is already expressed by the existing caps
-     * skip this structure */
-    if (i > 0 && gst_caps_is_subset_structure (res, st))
-      continue;
-
-    st = gst_structure_copy (st);
-    gst_structure_remove_field (st, "format");
-
-    /* Only remove the channels and channel-mask for non-NONE layouts */
-    if (gst_structure_get (st, "channel-mask", GST_TYPE_BITMASK, &channel_mask,
-            NULL)) {
-      if (channel_mask != 0)
-        remove_channels = TRUE;
-    } else {
-      remove_channels = TRUE;
-    }
-
-    if (remove_channels && channels)
-      gst_structure_remove_fields (st, "channel-mask", "channels", NULL);
-
-    gst_caps_append_structure (res, st);
+  /* Only remove the channels and channel-mask for non-NONE layouts,
+   * or if a mix matrix was manually specified */
+  if (this->mix_matrix_is_set ||
+      !gst_structure_get (s, "channel-mask", GST_TYPE_BITMASK, &mask, NULL) ||
+      (mask != 0 || (gst_structure_get_int (s, "channels", &channels)
+              && channels == 1))) {
+    gst_structure_remove_fields (s, "channel-mask", "channels", NULL);
   }
 
-  return res;
+  return TRUE;
+}
+
+static gboolean
+add_other_channels_to_structure (GstCapsFeatures * features, GstStructure * s,
+    gpointer user_data)
+{
+  gint other_channels = GPOINTER_TO_INT (user_data);
+
+  gst_structure_set (s, "channels", G_TYPE_INT, other_channels, NULL);
+
+  return TRUE;
 }
 
 /* The caps can be transformed into any other caps with format info removed.
@@ -291,9 +357,30 @@ gst_audio_convert_transform_caps (GstBaseTransform * btrans,
 {
   GstCaps *tmp, *tmp2;
   GstCaps *result;
+  GstAudioConvert *this = GST_AUDIO_CONVERT (btrans);
 
-  /* Get all possible caps that we can transform to */
-  tmp = gst_audio_convert_caps_remove_format_info (caps, TRUE);
+  tmp = gst_caps_copy (caps);
+
+  gst_caps_map_in_place (tmp, remove_format_from_structure, NULL);
+  gst_caps_map_in_place (tmp, remove_layout_from_structure, NULL);
+  gst_caps_map_in_place (tmp, remove_channels_from_structure, btrans);
+
+  /* We can infer the required input / output channels based on the
+   * matrix dimensions */
+  if (gst_value_array_get_size (&this->mix_matrix)) {
+    gint other_channels;
+
+    if (direction == GST_PAD_SRC) {
+      const GValue *first_row =
+          gst_value_array_get_value (&this->mix_matrix, 0);
+      other_channels = gst_value_array_get_size (first_row);
+    } else {
+      other_channels = gst_value_array_get_size (&this->mix_matrix);
+    }
+
+    gst_caps_map_in_place (tmp, add_other_channels_to_structure,
+        GINT_TO_POINTER (other_channels));
+  }
 
   if (filter) {
     tmp2 = gst_caps_intersect_full (filter, tmp, GST_CAPS_INTERSECT_FIRST);
@@ -608,12 +695,12 @@ gst_audio_convert_fixate_caps (GstBaseTransform * base,
 
   result = gst_caps_intersect (othercaps, caps);
   if (gst_caps_is_empty (result)) {
-    GstCaps *removed;
+    GstCaps *removed = gst_caps_copy (caps);
 
     if (result)
       gst_caps_unref (result);
-    /* try to preserve channels */
-    removed = gst_audio_convert_caps_remove_format_info (caps, FALSE);
+    gst_caps_map_in_place (removed, remove_format_from_structure, NULL);
+    gst_caps_map_in_place (removed, remove_layout_from_structure, NULL);
     result = gst_caps_intersect (othercaps, removed);
     gst_caps_unref (removed);
     if (gst_caps_is_empty (result)) {
@@ -654,6 +741,7 @@ gst_audio_convert_set_caps (GstBaseTransform * base, GstCaps * incaps,
   GstAudioInfo in_info;
   GstAudioInfo out_info;
   gboolean in_place;
+  GstStructure *config;
 
   GST_DEBUG_OBJECT (base, "incaps %" GST_PTR_FORMAT ", outcaps %"
       GST_PTR_FORMAT, incaps, outcaps);
@@ -668,12 +756,17 @@ gst_audio_convert_set_caps (GstBaseTransform * base, GstCaps * incaps,
   if (!gst_audio_info_from_caps (&out_info, outcaps))
     goto invalid_out;
 
-  this->convert = gst_audio_converter_new (0, &in_info, &out_info,
-      gst_structure_new ("GstAudioConverterConfig",
-          GST_AUDIO_CONVERTER_OPT_DITHER_METHOD, GST_TYPE_AUDIO_DITHER_METHOD,
-          this->dither,
-          GST_AUDIO_CONVERTER_OPT_NOISE_SHAPING_METHOD,
-          GST_TYPE_AUDIO_NOISE_SHAPING_METHOD, this->ns, NULL));
+  config = gst_structure_new ("GstAudioConverterConfig",
+      GST_AUDIO_CONVERTER_OPT_DITHER_METHOD, GST_TYPE_AUDIO_DITHER_METHOD,
+      this->dither,
+      GST_AUDIO_CONVERTER_OPT_NOISE_SHAPING_METHOD,
+      GST_TYPE_AUDIO_NOISE_SHAPING_METHOD, this->ns, NULL);
+
+  if (this->mix_matrix_is_set)
+    gst_structure_set_value (config, GST_AUDIO_CONVERTER_OPT_MIX_MATRIX,
+        &this->mix_matrix);
+
+  this->convert = gst_audio_converter_new (0, &in_info, &out_info, config);
 
   if (this->convert == NULL)
     goto no_converter;
@@ -681,6 +774,9 @@ gst_audio_convert_set_caps (GstBaseTransform * base, GstCaps * incaps,
   in_place = gst_audio_converter_supports_inplace (this->convert);
   gst_base_transform_set_in_place (base, in_place);
 
+  gst_base_transform_set_passthrough (base,
+      gst_audio_converter_is_passthrough (this->convert));
+
   this->in_info = in_info;
   this->out_info = out_info;
 
@@ -711,47 +807,28 @@ gst_audio_convert_transform (GstBaseTransform * base, GstBuffer * inbuf,
 {
   GstFlowReturn ret;
   GstAudioConvert *this = GST_AUDIO_CONVERT (base);
-  GstMapInfo srcmap = { NULL, }, dstmap;
-  gint insize, outsize;
+  GstAudioBuffer srcabuf, dstabuf;
   gboolean inbuf_writable;
   GstAudioConverterFlags flags;
-  gsize samples;
 
-  /* get amount of samples to convert. */
-  samples = gst_buffer_get_size (inbuf) / this->in_info.bpf;
-
-  /* get in/output sizes, to see if the buffers we got are of correct
-   * sizes */
-  insize = samples * this->in_info.bpf;
-  outsize = samples * this->out_info.bpf;
-
-  if (insize == 0 || outsize == 0)
+  /* https://bugzilla.gnome.org/show_bug.cgi?id=396835 */
+  if (gst_buffer_get_size (inbuf) == 0)
     return GST_FLOW_OK;
 
-  gst_buffer_resize (outbuf, 0, outsize);
-
-  /* get src and dst data */
   if (inbuf != outbuf) {
     inbuf_writable = gst_buffer_is_writable (inbuf)
         && gst_buffer_n_memory (inbuf) == 1
         && gst_memory_is_writable (gst_buffer_peek_memory (inbuf, 0));
 
-    if (!gst_buffer_map (inbuf, &srcmap,
+    if (!gst_audio_buffer_map (&srcabuf, &this->in_info, inbuf,
             inbuf_writable ? GST_MAP_READWRITE : GST_MAP_READ))
       goto inmap_error;
   } else {
     inbuf_writable = TRUE;
   }
-  if (!gst_buffer_map (outbuf, &dstmap, GST_MAP_WRITE))
-    goto outmap_error;
 
-  /* check in and outsize */
-  if (inbuf != outbuf) {
-    if (srcmap.size < insize)
-      goto wrong_size;
-  }
-  if (dstmap.size < outsize)
-    goto wrong_size;
+  if (!gst_audio_buffer_map (&dstabuf, &this->out_info, outbuf, GST_MAP_WRITE))
+    goto outmap_error;
 
   /* and convert the samples */
   flags = 0;
@@ -759,36 +836,28 @@ gst_audio_convert_transform (GstBaseTransform * base, GstBuffer * inbuf,
     flags |= GST_AUDIO_CONVERTER_FLAG_IN_WRITABLE;
 
   if (!GST_BUFFER_FLAG_IS_SET (inbuf, GST_BUFFER_FLAG_GAP)) {
-    gpointer in[1] = { srcmap.data };
-    gpointer out[1] = { dstmap.data };
-
     if (!gst_audio_converter_samples (this->convert, flags,
-            inbuf != outbuf ? in : out, samples, out, samples))
+            inbuf != outbuf ? srcabuf.planes : dstabuf.planes,
+            dstabuf.n_samples, dstabuf.planes, dstabuf.n_samples))
       goto convert_error;
   } else {
     /* Create silence buffer */
-    gst_audio_format_fill_silence (this->out_info.finfo, dstmap.data, outsize);
+    gint i;
+    for (i = 0; i < dstabuf.n_planes; i++) {
+      gst_audio_format_fill_silence (this->out_info.finfo, dstabuf.planes[i],
+          GST_AUDIO_BUFFER_PLANE_SIZE (&dstabuf));
+    }
   }
   ret = GST_FLOW_OK;
 
 done:
-  gst_buffer_unmap (outbuf, &dstmap);
+  gst_audio_buffer_unmap (&dstabuf);
   if (inbuf != outbuf)
-    gst_buffer_unmap (inbuf, &srcmap);
+    gst_audio_buffer_unmap (&srcabuf);
 
   return ret;
 
   /* ERRORS */
-wrong_size:
-  {
-    GST_ELEMENT_ERROR (this, STREAM, FORMAT,
-        (NULL),
-        ("input/output buffers are of wrong size in: %" G_GSIZE_FORMAT " < %d"
-            " or out: %" G_GSIZE_FORMAT " < %d",
-            srcmap.size, insize, dstmap.size, outsize));
-    ret = GST_FLOW_ERROR;
-    goto done;
-  }
 convert_error:
   {
     GST_ELEMENT_ERROR (this, STREAM, FORMAT,
@@ -807,7 +876,7 @@ outmap_error:
     GST_ELEMENT_ERROR (this, STREAM, FORMAT,
         (NULL), ("failed to map output buffer"));
     if (inbuf != outbuf)
-      gst_buffer_unmap (inbuf, &srcmap);
+      gst_audio_buffer_unmap (&srcabuf);
     return GST_FLOW_ERROR;
   }
 }
@@ -854,6 +923,43 @@ gst_audio_convert_submit_input_buffer (GstBaseTransform * base,
       is_discont, input);
 }
 
+static GstFlowReturn
+gst_audio_convert_prepare_output_buffer (GstBaseTransform * base,
+    GstBuffer * inbuf, GstBuffer ** outbuf)
+{
+  GstAudioConvert *this = GST_AUDIO_CONVERT (base);
+  GstAudioMeta *meta;
+  GstFlowReturn ret;
+
+  ret = GST_BASE_TRANSFORM_CLASS (parent_class)->prepare_output_buffer (base,
+      inbuf, outbuf);
+
+  if (ret != GST_FLOW_OK)
+    return ret;
+
+  meta = gst_buffer_get_audio_meta (inbuf);
+
+  if (inbuf != *outbuf) {
+    gsize samples = meta ?
+        meta->samples : (gst_buffer_get_size (inbuf) / this->in_info.bpf);
+
+    /* ensure that the output buffer is not bigger than what we need */
+    gst_buffer_resize (*outbuf, 0, samples * this->out_info.bpf);
+
+    /* add the audio meta on the output buffer if it's planar */
+    if (this->out_info.layout == GST_AUDIO_LAYOUT_NON_INTERLEAVED) {
+      gst_buffer_add_audio_meta (*outbuf, &this->out_info, samples, NULL);
+    }
+  } else {
+    /* if the input buffer came with a GstAudioMeta,
+     * update it to reflect the properties of the output format */
+    if (meta)
+      meta->info = this->out_info;
+  }
+
+  return ret;
+}
+
 static void
 gst_audio_convert_set_property (GObject * object, guint prop_id,
     const GValue * value, GParamSpec * pspec)
@@ -867,6 +973,23 @@ gst_audio_convert_set_property (GObject * object, guint prop_id,
     case PROP_NOISE_SHAPING:
       this->ns = g_value_get_enum (value);
       break;
+    case PROP_MIX_MATRIX:
+      if (!gst_value_array_get_size (value)) {
+        this->mix_matrix_is_set = FALSE;
+      } else {
+        const GValue *first_row = gst_value_array_get_value (value, 0);
+
+        if (gst_value_array_get_size (first_row)) {
+          g_value_copy (value, &this->mix_matrix);
+          this->mix_matrix_is_set = TRUE;
+
+          /* issue a reconfigure upstream */
+          gst_base_transform_reconfigure_sink (GST_BASE_TRANSFORM (this));
+        } else {
+          g_warning ("Empty mix matrix's first row");
+        }
+      }
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -886,6 +1009,10 @@ gst_audio_convert_get_property (GObject * object, guint prop_id,
     case PROP_NOISE_SHAPING:
       g_value_set_enum (value, this->ns);
       break;
+    case PROP_MIX_MATRIX:
+      if (this->mix_matrix_is_set)
+        g_value_copy (&this->mix_matrix, value);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
