@@ -21,6 +21,7 @@
 
 /**
  * SECTION:element-audioresample
+ * @title: audioresample
  *
  * audioresample resamples raw audio buffers to different sample rates using
  * a configurable windowing function to enhance quality.
@@ -33,14 +34,14 @@
  * to initialize when the element is created. A third mode exists, which uses the full table
  * unless said table would become too large, in which case the interpolated one is used instead.
  *
- * <refsect2>
- * <title>Example launch line</title>
+ * ## Example launch line
  * |[
  * gst-launch-1.0 -v uridecodebin uri=file:///path/to/audio.ogg ! audioconvert ! audioresample ! audio/x-raw, rate=8000 ! autoaudiosink
- * ]| Decode an audio file and downsample it to 8Khz and play sound.
+ * ]|
+ *  Decode an audio file and downsample it to 8Khz and play sound.
  * To create the Ogg/Vorbis file refer to the documentation of vorbisenc.
  * This assumes there is an audio sink that will accept/handle 8kHz audio.
- * </refsect2>
+ *
  */
 
 /* TODO:
@@ -61,9 +62,6 @@
 
 GST_DEBUG_CATEGORY (audio_resample_debug);
 #define GST_CAT_DEFAULT audio_resample_debug
-#if !defined(AUDIORESAMPLE_FORMAT_AUTO)
-GST_DEBUG_CATEGORY_STATIC (GST_CAT_PERFORMANCE);
-#endif
 
 #undef USE_SPEEX
 
@@ -129,6 +127,9 @@ static gboolean gst_audio_resample_stop (GstBaseTransform * base);
 static gboolean gst_audio_resample_query (GstPad * pad, GstObject * parent,
     GstQuery * query);
 
+static void gst_audio_resample_push_drain (GstAudioResample * resample,
+    guint history_len);
+
 #define gst_audio_resample_parent_class parent_class
 G_DEFINE_TYPE (GstAudioResample, gst_audio_resample, GST_TYPE_BASE_TRANSFORM);
 
@@ -209,6 +210,11 @@ gst_audio_resample_class_init (GstAudioResampleClass * klass)
       GST_DEBUG_FUNCPTR (gst_audio_resample_submit_input_buffer);
 
   GST_BASE_TRANSFORM_CLASS (klass)->passthrough_on_same_caps = TRUE;
+
+  gst_type_mark_as_plugin_api (GST_TYPE_AUDIO_RESAMPLER_METHOD, 0);
+  gst_type_mark_as_plugin_api (GST_TYPE_AUDIO_RESAMPLER_FILTER_INTERPOLATION,
+      0);
+  gst_type_mark_as_plugin_api (GST_TYPE_AUDIO_RESAMPLER_FILTER_MODE, 0);
 }
 
 static void
@@ -289,7 +295,7 @@ gst_audio_resample_transform_caps (GstBaseTransform * base,
 
   /* transform single caps into input_caps + input_caps with the rate
    * field set to our supported range. This ensures that upstream knows
-   * about downstream's prefered rate(s) and can negotiate accordingly. */
+   * about downstream's preferred rate(s) and can negotiate accordingly. */
   res = gst_caps_new_empty ();
   n = gst_caps_get_size (caps);
   for (i = 0; i < n; i++) {
@@ -499,7 +505,27 @@ gst_audio_resample_set_caps (GstBaseTransform * base, GstCaps * incaps,
   if (!gst_audio_info_from_caps (&out, outcaps))
     goto invalid_outcaps;
 
-  /* FIXME do some checks */
+  /* Reset timestamp tracking and drain the resampler if the audio format is
+   * changing. Especially when changing the sample rate our timestamp tracking
+   * will be completely off, but even otherwise we would usually lose the last
+   * few samples if we don't drain here */
+  if (!gst_audio_info_is_equal (&in, &resample->in) ||
+      !gst_audio_info_is_equal (&out, &resample->out)) {
+    if (resample->converter) {
+      gsize latency = gst_audio_converter_get_max_latency (resample->converter);
+      gst_audio_resample_push_drain (resample, latency);
+    }
+    gst_audio_resample_reset_state (resample);
+    resample->num_gap_samples = 0;
+    resample->num_nongap_samples = 0;
+    resample->t0 = GST_CLOCK_TIME_NONE;
+    resample->in_offset0 = GST_BUFFER_OFFSET_NONE;
+    resample->out_offset0 = GST_BUFFER_OFFSET_NONE;
+    resample->samples_in = 0;
+    resample->samples_out = 0;
+    resample->need_discont = TRUE;
+  }
+
   gst_audio_resample_update_state (resample, &in, &out);
 
   resample->in = in;
@@ -547,8 +573,7 @@ gst_audio_resample_push_drain (GstAudioResample * resample, guint history_len)
   GstFlowReturn res;
   gint outsize;
   gsize out_len;
-  GstMapInfo map;
-  gpointer out[1];
+  GstAudioBuffer abuf;
 
   g_assert (resample->converter != NULL);
 
@@ -564,13 +589,15 @@ gst_audio_resample_push_drain (GstAudioResample * resample, guint history_len)
   outsize = out_len * resample->in.bpf;
   outbuf = gst_buffer_new_and_alloc (outsize);
 
-  gst_buffer_map (outbuf, &map, GST_MAP_WRITE);
+  if (GST_AUDIO_INFO_LAYOUT (&resample->out) ==
+      GST_AUDIO_LAYOUT_NON_INTERLEAVED) {
+    gst_buffer_add_audio_meta (outbuf, &resample->out, out_len, NULL);
+  }
 
-  out[0] = map.data;
+  gst_audio_buffer_map (&abuf, &resample->out, outbuf, GST_MAP_WRITE);
   gst_audio_converter_samples (resample->converter, 0, NULL, history_len,
-      out, out_len);
-
-  gst_buffer_unmap (outbuf, &map);
+      abuf.planes, out_len);
+  gst_audio_buffer_unmap (&abuf);
 
   /* time */
   if (GST_CLOCK_TIME_IS_VALID (resample->t0)) {
@@ -630,7 +657,9 @@ gst_audio_resample_sink_event (GstBaseTransform * base, GstEvent * event)
       resample->samples_out = 0;
       resample->need_discont = TRUE;
       break;
+    case GST_EVENT_STREAM_START:
     case GST_EVENT_SEGMENT:
+    case GST_EVENT_EOS:
       if (resample->converter) {
         gsize latency =
             gst_audio_converter_get_max_latency (resample->converter);
@@ -646,14 +675,6 @@ gst_audio_resample_sink_event (GstBaseTransform * base, GstEvent * event)
       resample->samples_out = 0;
       resample->need_discont = TRUE;
       break;
-    case GST_EVENT_EOS:
-      if (resample->converter) {
-        gsize latency =
-            gst_audio_converter_get_max_latency (resample->converter);
-        gst_audio_resample_push_drain (resample, latency);
-      }
-      gst_audio_resample_reset_state (resample);
-      break;
     default:
       break;
   }
@@ -702,10 +723,10 @@ static GstFlowReturn
 gst_audio_resample_process (GstAudioResample * resample, GstBuffer * inbuf,
     GstBuffer * outbuf)
 {
-  GstMapInfo in_map, out_map;
+  GstAudioBuffer srcabuf, dstabuf;
   gsize outsize;
-  guint32 in_len;
-  guint32 out_len;
+  gsize in_len;
+  gsize out_len;
   guint filt_len =
       gst_audio_converter_get_max_latency (resample->converter) * 2;
   gboolean inbuf_writable;
@@ -714,12 +735,21 @@ gst_audio_resample_process (GstAudioResample * resample, GstBuffer * inbuf,
       && gst_buffer_n_memory (inbuf) == 1
       && gst_memory_is_writable (gst_buffer_peek_memory (inbuf, 0));
 
-  gst_buffer_map (inbuf, &in_map,
+  gst_audio_buffer_map (&srcabuf, &resample->in, inbuf,
       inbuf_writable ? GST_MAP_READWRITE : GST_MAP_READ);
-  gst_buffer_map (outbuf, &out_map, GST_MAP_WRITE);
 
-  in_len = in_map.size / resample->in.bpf;
-  out_len = out_map.size / resample->out.bpf;
+  in_len = srcabuf.n_samples;
+  out_len = gst_audio_converter_get_out_frames (resample->converter, in_len);
+
+  /* ensure that the output buffer is not bigger than what we need */
+  gst_buffer_set_size (outbuf, out_len * resample->in.bpf);
+
+  if (GST_AUDIO_INFO_LAYOUT (&resample->out) ==
+      GST_AUDIO_LAYOUT_NON_INTERLEAVED) {
+    gst_buffer_add_audio_meta (outbuf, &resample->out, out_len, NULL);
+  }
+
+  gst_audio_buffer_map (&dstabuf, &resample->out, outbuf, GST_MAP_WRITE);
 
   if (GST_BUFFER_FLAG_IS_SET (inbuf, GST_BUFFER_FLAG_GAP)) {
     resample->num_nongap_samples = 0;
@@ -737,6 +767,7 @@ gst_audio_resample_process (GstAudioResample * resample, GstBuffer * inbuf,
 
     {
       guint num, den;
+      gint i;
 
       num = resample->in.rate;
       den = resample->out.rate;
@@ -748,7 +779,9 @@ gst_audio_resample_process (GstAudioResample * resample, GstBuffer * inbuf,
       else
         out_len = 0;
 
-      memset (out_map.data, 0, out_map.size);
+      for (i = 0; i < dstabuf.n_planes; i++)
+        memset (dstabuf.planes[i], 0, GST_AUDIO_BUFFER_PLANE_SIZE (&dstabuf));
+
       GST_BUFFER_FLAG_SET (outbuf, GST_BUFFER_FLAG_GAP);
       resample->num_gap_samples += in_len;
     }
@@ -770,17 +803,14 @@ gst_audio_resample_process (GstAudioResample * resample, GstBuffer * inbuf,
     }
     {
       /* process */
-      gpointer in[1], out[1];
       GstAudioConverterFlags flags;
 
       flags = 0;
       if (inbuf_writable)
         flags |= GST_AUDIO_CONVERTER_FLAG_IN_WRITABLE;
 
-      in[0] = in_map.data;
-      out[0] = out_map.data;
-      gst_audio_converter_samples (resample->converter, flags, in, in_len,
-          out, out_len);
+      gst_audio_converter_samples (resample->converter, flags, srcabuf.planes,
+          in_len, dstabuf.planes, out_len);
     }
   }
 
@@ -808,13 +838,13 @@ gst_audio_resample_process (GstAudioResample * resample, GstBuffer * inbuf,
   resample->samples_out += out_len;
   resample->samples_in += in_len;
 
-  gst_buffer_unmap (inbuf, &in_map);
-  gst_buffer_unmap (outbuf, &out_map);
+  gst_audio_buffer_unmap (&srcabuf);
+  gst_audio_buffer_unmap (&dstabuf);
 
   outsize = out_len * resample->in.bpf;
 
   GST_LOG_OBJECT (resample,
-      "Converted to buffer of %" G_GUINT32_FORMAT
+      "Converted to buffer of %" G_GSIZE_FORMAT
       " samples (%" G_GSIZE_FORMAT " bytes) with timestamp %" GST_TIME_FORMAT
       ", duration %" GST_TIME_FORMAT ", offset %" G_GUINT64_FORMAT
       ", offset_end %" G_GUINT64_FORMAT, out_len, outsize,
@@ -846,16 +876,13 @@ gst_audio_resample_transform (GstBaseTransform * base, GstBuffer * inbuf,
    * flag to resync timestamp and offset counters and send event
    * downstream */
   if (G_UNLIKELY (gst_audio_resample_check_discont (resample, inbuf))) {
-    gsize size;
-    gint bpf = GST_AUDIO_INFO_BPF (&resample->in);
+    if (resample->converter) {
+      gsize latency = gst_audio_converter_get_max_latency (resample->converter);
+      gst_audio_resample_push_drain (resample, latency);
+    }
 
     gst_audio_resample_reset_state (resample);
     resample->need_discont = TRUE;
-
-    /* need to recalculate the output size */
-    size = gst_buffer_get_size (inbuf) / bpf;
-    size = gst_audio_converter_get_out_frames (resample->converter, size);
-    gst_buffer_set_size (outbuf, size * bpf);
   }
 
   /* handle discontinuity */
