@@ -22,6 +22,7 @@
 #include "config.h"
 #endif
 
+#include <gst/video/video.h>
 #include "gststreamsplitter.h"
 
 static GstStaticPadTemplate src_template =
@@ -66,10 +67,8 @@ gst_stream_splitter_class_init (GstStreamSplitterClass * klass)
   GST_DEBUG_CATEGORY_INIT (gst_stream_splitter_debug, "streamsplitter", 0,
       "Stream Splitter");
 
-  gst_element_class_add_pad_template (gstelement_klass,
-      gst_static_pad_template_get (&src_template));
-  gst_element_class_add_pad_template (gstelement_klass,
-      gst_static_pad_template_get (&sink_template));
+  gst_element_class_add_static_pad_template (gstelement_klass, &src_template);
+  gst_element_class_add_static_pad_template (gstelement_klass, &sink_template);
 
   gstelement_klass->request_new_pad =
       GST_DEBUG_FUNCPTR (gst_stream_splitter_request_new_pad);
@@ -155,13 +154,12 @@ _flush_events (GstPad * pad, GList * events)
   GList *tmp;
 
   for (tmp = events; tmp; tmp = tmp->next) {
-    if (GST_EVENT_TYPE (tmp->data) == GST_EVENT_EOS ||
-        GST_EVENT_TYPE (tmp->data) == GST_EVENT_SEGMENT ||
-        !GST_EVENT_IS_STICKY (tmp->data) || pad == NULL) {
-      gst_event_unref (tmp->data);
-    } else {
+    if (GST_EVENT_TYPE (tmp->data) != GST_EVENT_EOS &&
+        GST_EVENT_TYPE (tmp->data) != GST_EVENT_SEGMENT &&
+        GST_EVENT_IS_STICKY (tmp->data) && pad != NULL) {
       gst_pad_store_sticky_event (pad, GST_EVENT_CAST (tmp->data));
     }
+    gst_event_unref (tmp->data);
   }
   g_list_free (events);
 
@@ -467,6 +465,29 @@ beach:
   return res;
 }
 
+static gboolean
+gst_stream_splitter_src_event (GstPad * pad, GstObject * parent,
+    GstEvent * event)
+{
+  GstStreamSplitter *stream_splitter = (GstStreamSplitter *) parent;
+
+  if (gst_video_event_is_force_key_unit (event)) {
+    guint32 seqnum = gst_event_get_seqnum (event);
+    STREAMS_LOCK (stream_splitter);
+    if (seqnum == stream_splitter->keyunit_seqnum) {
+      STREAMS_UNLOCK (stream_splitter);
+      GST_TRACE_OBJECT (pad,
+          "Drop duplicated force-key-unit event %" G_GUINT32_FORMAT, seqnum);
+      gst_event_unref (event);
+      return TRUE;
+    }
+    stream_splitter->keyunit_seqnum = seqnum;
+    STREAMS_UNLOCK (stream_splitter);
+  }
+
+  return gst_pad_event_default (pad, parent, event);
+}
+
 static void
 gst_stream_splitter_init (GstStreamSplitter * stream_splitter)
 {
@@ -497,6 +518,7 @@ gst_stream_splitter_request_new_pad (GstElement * element,
   gst_pad_set_active (srcpad, TRUE);
   gst_element_add_pad (element, srcpad);
   stream_splitter->cookie++;
+  gst_pad_set_event_function (srcpad, gst_stream_splitter_src_event);
   STREAMS_UNLOCK (stream_splitter);
 
   return srcpad;
