@@ -91,8 +91,8 @@
  *   which contains caps from the updated configuration.
  *
  * * In case there are bytes in each frame that aren't part of the actual
- *   payload, the `get_overhead_size()` vfunc must be defined, and the
- *   `get_config_frame_size()` vfunc must return a frame size that includes
+ *   payload, the get_overhead_size() vfunc must be defined, and the
+ *   @get_config_frame_size() vfunc must return a frame size that includes
  *   the number of non-payload bytes (= the overhead). Otherwise, the
  *   timestamps will incorrectly include the overhead bytes.
  */
@@ -182,8 +182,6 @@ gst_raw_base_parse_class_init (GstRawBaseParseClass * klass)
           "Use the sink caps for defining the output format",
           DEFAULT_USE_SINK_CAPS, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS)
       );
-
-  gst_type_mark_as_plugin_api (GST_TYPE_RAW_BASE_PARSE, 0);
 }
 
 static void
@@ -700,8 +698,7 @@ gst_raw_base_parse_convert (GstBaseParse * parse, GstFormat src_format,
     *dest_value = src_value;
   } else if ((src_format == GST_FORMAT_TIME || dest_format == GST_FORMAT_TIME)
       && gst_raw_base_parse_is_gstformat_supported (raw_base_parse, src_format)
-      && gst_raw_base_parse_is_gstformat_supported (raw_base_parse,
-          dest_format)) {
+      && gst_raw_base_parse_is_gstformat_supported (raw_base_parse, src_format)) {
     /* Perform conversions here if either the src or dest format
      * are GST_FORMAT_TIME and the other format is supported by
      * the subclass. This is because we perform TIME<->non-TIME
