@@ -20,6 +20,7 @@
 
 /**
  * SECTION:element-videoscale
+ * @title: videoscale
  * @see_also: videorate, videoconvert
  *
  * This element resizes video frames. By default the element will try to
@@ -31,18 +32,19 @@
  * RGB formats and is therefore generally able to operate anywhere in a
  * pipeline.
  *
- * <refsect2>
- * <title>Example pipelines</title>
+ * ## Example pipelines
  * |[
  * gst-launch-1.0 -v filesrc location=videotestsrc.ogg ! oggdemux ! theoradec ! videoconvert ! videoscale ! autovideosink
- * ]| Decode an Ogg/Theora and display the video. If the video sink chosen
+ * ]|
+ *  Decode an Ogg/Theora and display the video. If the video sink chosen
  * cannot perform scaling, the video scaling will be performed by videoscale
  * when you resize the video window.
  * To create the test Ogg/Theora file refer to the documentation of theoraenc.
  * |[
  * gst-launch-1.0 -v filesrc location=videotestsrc.ogg ! oggdemux ! theoradec ! videoconvert ! videoscale ! video/x-raw,width=100 ! autovideosink
- * ]| Decode an Ogg/Theora and display the video with a width of 100.
- * </refsect2>
+ * ]|
+ *  Decode an Ogg/Theora and display the video with a width of 100.
+ *
  */
 
 /* 
@@ -125,6 +127,9 @@ static GstStaticCaps gst_video_scale_format_caps =
     GST_STATIC_CAPS (GST_VIDEO_CAPS_MAKE (GST_VIDEO_FORMATS) ";"
     GST_VIDEO_CAPS_MAKE_WITH_FEATURES ("ANY", GST_VIDEO_FORMATS));
 
+GQuark _size_quark;
+GQuark _scale_quark;
+
 #define GST_TYPE_VIDEO_SCALE_METHOD (gst_video_scale_method_get_type())
 static GType
 gst_video_scale_method_get_type (void)
@@ -189,6 +194,8 @@ static GstCaps *gst_video_scale_transform_caps (GstBaseTransform * trans,
     GstPadDirection direction, GstCaps * caps, GstCaps * filter);
 static GstCaps *gst_video_scale_fixate_caps (GstBaseTransform * base,
     GstPadDirection direction, GstCaps * caps, GstCaps * othercaps);
+static gboolean gst_video_scale_transform_meta (GstBaseTransform * trans,
+    GstBuffer * inbuf, GstMeta * meta, GstBuffer * outbuf);
 
 static gboolean gst_video_scale_set_info (GstVideoFilter * filter,
     GstCaps * in, GstVideoInfo * in_info, GstCaps * out,
@@ -281,10 +288,17 @@ gst_video_scale_class_init (GstVideoScaleClass * klass)
       GST_DEBUG_FUNCPTR (gst_video_scale_transform_caps);
   trans_class->fixate_caps = GST_DEBUG_FUNCPTR (gst_video_scale_fixate_caps);
   trans_class->src_event = GST_DEBUG_FUNCPTR (gst_video_scale_src_event);
+  trans_class->transform_meta =
+      GST_DEBUG_FUNCPTR (gst_video_scale_transform_meta);
 
   filter_class->set_info = GST_DEBUG_FUNCPTR (gst_video_scale_set_info);
   filter_class->transform_frame =
       GST_DEBUG_FUNCPTR (gst_video_scale_transform_frame);
+
+  _size_quark = g_quark_from_static_string (GST_META_TAG_VIDEO_SIZE_STR);
+  _scale_quark = gst_video_meta_transform_scale_get_quark ();
+
+  gst_type_mark_as_plugin_api (GST_TYPE_VIDEO_SCALE_METHOD, 0);
 }
 
 static void
@@ -485,6 +499,57 @@ gst_video_scale_transform_caps (GstBaseTransform * trans,
   return ret;
 }
 
+static gboolean
+gst_video_scale_transform_meta (GstBaseTransform * trans, GstBuffer * inbuf,
+    GstMeta * meta, GstBuffer * outbuf)
+{
+  GstVideoFilter *videofilter = GST_VIDEO_FILTER (trans);
+  const GstMetaInfo *info = meta->info;
+  const gchar *const *tags;
+  const gchar *const *curr = NULL;
+  gboolean should_copy = TRUE;
+  const gchar *const valid_tags[] = { GST_META_TAG_VIDEO_STR,
+    GST_META_TAG_VIDEO_COLORSPACE_STR,
+    GST_META_TAG_VIDEO_ORIENTATION_STR,
+    GST_META_TAG_VIDEO_SIZE_STR
+  };
+
+  tags = gst_meta_api_type_get_tags (info->api);
+
+  /* No specific tags, we are good to copy */
+  if (!tags) {
+    return TRUE;
+  }
+
+  /* We are only changing size, we can preserve other metas tagged as
+     orientation and colorspace */
+  for (curr = tags; *curr; ++curr) {
+
+    /* We dont handle any other tag */
+    if (!g_strv_contains (valid_tags, *curr)) {
+      should_copy = FALSE;
+      break;
+    }
+  }
+
+  /* Cant handle the tags in this meta, let the parent class handle it */
+  if (!should_copy) {
+    return GST_BASE_TRANSFORM_CLASS (parent_class)->transform_meta (trans,
+        inbuf, meta, outbuf);
+  }
+
+  /* This meta is size sensitive, try to transform it accordingly */
+  if (gst_meta_api_type_has_tag (info->api, _size_quark)) {
+    GstVideoMetaTransform trans =
+        { &videofilter->in_info, &videofilter->out_info };
+
+    return info->transform_func (outbuf, meta, inbuf, _scale_quark, &trans);
+  }
+
+  /* No need to transform, we can safely copy this meta */
+  return TRUE;
+}
+
 static gboolean
 gst_video_scale_set_info (GstVideoFilter * filter, GstCaps * in,
     GstVideoInfo * in_info, GstCaps * out, GstVideoInfo * out_info)
@@ -656,8 +721,8 @@ gst_video_scale_fixate_caps (GstBaseTransform * base, GstPadDirection direction,
 {
   GstStructure *ins, *outs;
   const GValue *from_par, *to_par;
-  GValue fpar = { 0, }, tpar = {
-  0,};
+  GValue fpar = { 0, };
+  GValue tpar = { 0, };
 
   othercaps = gst_caps_truncate (othercaps);
   othercaps = gst_caps_make_writable (othercaps);
@@ -779,7 +844,7 @@ gst_video_scale_fixate_caps (GstBaseTransform * base, GstPadDirection direction,
           goto done;
         }
 
-        w = (guint) gst_util_uint64_scale_int (h, num, den);
+        w = (guint) gst_util_uint64_scale_int_round (h, num, den);
         gst_structure_fixate_field_nearest_int (outs, "width", w);
 
         goto done;
@@ -831,7 +896,7 @@ gst_video_scale_fixate_caps (GstBaseTransform * base, GstPadDirection direction,
         goto done;
       }
 
-      w = (guint) gst_util_uint64_scale_int (h, num, den);
+      w = (guint) gst_util_uint64_scale_int_round (h, num, den);
       gst_structure_fixate_field_nearest_int (outs, "width", w);
       if (gst_structure_has_field (outs, "pixel-aspect-ratio") ||
           set_par_n != set_par_d)
@@ -861,7 +926,7 @@ gst_video_scale_fixate_caps (GstBaseTransform * base, GstPadDirection direction,
           goto done;
         }
 
-        h = (guint) gst_util_uint64_scale_int (w, den, num);
+        h = (guint) gst_util_uint64_scale_int_round (w, den, num);
         gst_structure_fixate_field_nearest_int (outs, "height", h);
 
         goto done;
@@ -912,7 +977,7 @@ gst_video_scale_fixate_caps (GstBaseTransform * base, GstPadDirection direction,
         goto done;
       }
 
-      h = (guint) gst_util_uint64_scale_int (w, den, num);
+      h = (guint) gst_util_uint64_scale_int_round (w, den, num);
       gst_structure_fixate_field_nearest_int (outs, "height", h);
       if (gst_structure_has_field (outs, "pixel-aspect-ratio") ||
           set_par_n != set_par_d)
@@ -942,7 +1007,7 @@ gst_video_scale_fixate_caps (GstBaseTransform * base, GstPadDirection direction,
 
       /* This might have failed but try to scale the width
        * to keep the DAR nonetheless */
-      w = (guint) gst_util_uint64_scale_int (set_h, num, den);
+      w = (guint) gst_util_uint64_scale_int_round (set_h, num, den);
       gst_structure_fixate_field_nearest_int (tmp, "width", w);
       gst_structure_get_int (tmp, "width", &set_w);
       gst_structure_free (tmp);
@@ -964,7 +1029,7 @@ gst_video_scale_fixate_caps (GstBaseTransform * base, GstPadDirection direction,
 
       /* This might have failed but try to scale the width
        * to keep the DAR nonetheless */
-      h = (guint) gst_util_uint64_scale_int (set_w, den, num);
+      h = (guint) gst_util_uint64_scale_int_round (set_w, den, num);
       gst_structure_fixate_field_nearest_int (tmp, "height", h);
       gst_structure_get_int (tmp, "height", &set_h);
       gst_structure_free (tmp);
@@ -976,10 +1041,14 @@ gst_video_scale_fixate_caps (GstBaseTransform * base, GstPadDirection direction,
         goto done;
       }
 
-      /* If all this failed, keep the height that was nearest to the orignal
-       * height and the nearest possible width. This changes the DAR but
-       * there's not much else to do here.
+      /* If all this failed, keep the dimensions with the DAR that was closest
+       * to the correct DAR. This changes the DAR but there's not much else to
+       * do here.
        */
+      if (set_w * ABS (set_h - h) < ABS (f_w - w) * f_h) {
+        f_h = set_h;
+        f_w = set_w;
+      }
       gst_structure_set (outs, "width", G_TYPE_INT, f_w, "height", G_TYPE_INT,
           f_h, NULL);
       goto done;
@@ -1033,7 +1102,7 @@ gst_video_scale_fixate_caps (GstBaseTransform * base, GstPadDirection direction,
         goto done;
       }
 
-      w = (guint) gst_util_uint64_scale_int (set_h, num, den);
+      w = (guint) gst_util_uint64_scale_int_round (set_h, num, den);
       tmp = gst_structure_copy (outs);
       gst_structure_fixate_field_nearest_int (tmp, "width", w);
       gst_structure_get_int (tmp, "width", &tmp2);
@@ -1050,7 +1119,7 @@ gst_video_scale_fixate_caps (GstBaseTransform * base, GstPadDirection direction,
       }
 
       /* ... or try the same with the height */
-      h = (guint) gst_util_uint64_scale_int (set_w, den, num);
+      h = (guint) gst_util_uint64_scale_int_round (set_w, den, num);
       tmp = gst_structure_copy (outs);
       gst_structure_fixate_field_nearest_int (tmp, "height", h);
       gst_structure_get_int (tmp, "height", &tmp2);
